/*
 * Created on Apr 17, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package Viz;

/**
 * @author Lior Berry
 * UBC
*/

import java.awt.*;
import javax.swing.*;
import java.awt.image.BufferedImage;
import java.util.*;
import java.awt.event.*;
import java.text.NumberFormat;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;

import DataProcess.*;
import Utils.ClusterAlgo;
import Utils.GraphicUtils;

/**
 * @author Lior Berry
 * UBC
*/
public class ClusterVizPanel extends JPanel implements ActionListener,MouseListener,MouseMotionListener,KeyListener, ComponentListener {

	protected ClusterAlgo		mClusterAlgo = null;
	protected int	 			mClusterInView = 0;
	protected VizPanelMgr		mVizMgr = null;

	private BufferedImage bimg 		= null;
	private BufferedImage finalImg 	= null;
	
	protected Rectangle mDisplayRegion;

	private int dotSize = 3;
	private int halfDotSize = 1;
	private int mLeapX = -1;
	private ClusterAlgo.Cluster mViewdCluster = null;
	
	protected int mIntervalShift = 0;
	
	private static String BTN_LEFT 	= "BTN_LEFT";
	private static String BTN_RIGHT = "BTN_RIGHT";
	
	protected JButton leftButton = null;
	protected JButton rightButton = null;
	
	protected JDialog mDlg = null;
	
	protected int mSavedHeight = -1;
	
	private boolean mDrawTickLabels = true;
	private int		mTickLabelFontSize = 10;

	

	void SetClusterAlgo(ClusterAlgo clusterAlgo)
	{
		mClusterAlgo = 	clusterAlgo;
	} 
	
	private Graphics2D createGraphics2D(int w, int h) {
		
		Graphics2D g2 = null;
		

		try{		
		
		if (h < 1){
			h = 1;
		}
		
		if (w < 1)
			w = 1;
		
		boolean needRecreation = false;
		if (bimg == null || bimg.getWidth() != w || bimg.getHeight() != h) {
			needRecreation = true;
		} 
		if (finalImg == null || finalImg.getWidth() != w || finalImg.getHeight() != h) {
			needRecreation = true;
		} 

		if (needRecreation){
			bimg = (BufferedImage) createImage(w, h);
			finalImg = (BufferedImage) createImage(w, h);;
		}

		g2 = bimg.createGraphics();
		g2.setBackground(Color.white);
		g2.clearRect(0, 0, w, h);
		}
		
		catch(java.lang.NegativeArraySizeException e){
			System.out.println(e.toString());
			System.out.println(w + "," + h);	
			
		}
		
		return g2;
	}	
	
	public void paintComponent(Graphics gfx) {
		
		super.paintComponent(gfx);

		Graphics2D g = (Graphics2D) gfx;
		PaintBufferedImg();
		g.drawImage(finalImg, 0, 0, this);

	}
	
	public void RegisterWithVizPanelMgr(VizPanelMgr inMgr)
	{
		mVizMgr = inMgr;
	}
	
	public void Init()
	{
		Dimension d = getSize();
		mDisplayRegion = new Rectangle(40,0,d.width-10,d.height-30);

		this.setLayout(null);
		rightButton = new JButton();
		this.add(rightButton);
		rightButton.setLocation(10,d.height-10);
		rightButton.setSize(10,10);
		java.net.URL imgURL;
		imgURL = UI.UIControls.class.getResource("small_arrow_right.gif");
		if (imgURL != null)
		rightButton.setIcon(new ImageIcon(imgURL));
		rightButton.setActionCommand(BTN_RIGHT);
		rightButton.addActionListener(this);

		leftButton = new JButton();
		this.add(leftButton);
		leftButton.setLocation(0,d.height-10);
		leftButton.setSize(10,10);
		imgURL = UI.UIControls.class.getResource("small_arrow_left.gif");
		if (imgURL != null)
		leftButton.setIcon(new ImageIcon(imgURL));
		leftButton.setActionCommand(BTN_LEFT);
		leftButton.addActionListener(this);
		
		addKeyListener(this);
		addMouseListener(this);
		addMouseMotionListener(this);
	}		

	private void PaintBufferedImg()
	{
		Dimension d = getSize();
		
		mViewdCluster = null;
		
		Graphics2D g2 = createGraphics2D(d.width, d.height-10);
		
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				   RenderingHints.VALUE_ANTIALIAS_ON);
	
		boolean drawBorder = true;
		if (drawBorder){
			g2.setColor(Color.black);
			g2.drawRect(0,0,d.width-1,d.height-1-10);
		}
		BinData binVals = null;
		
		Vector clusterList = null;
		if (mClusterAlgo != null){
			clusterList = mClusterAlgo.ExtractClusters();
		}
	
		if (clusterList != null && mClusterInView >= clusterList.size()){
			mClusterInView = 0;
		}
	
		if (clusterList != null && mClusterInView >=0 && mClusterInView < clusterList.size()){
			
			g2.setColor(GraphicUtils.GetColorForCluster(mClusterInView));
			ClusterAlgo.Cluster auxCluster = (ClusterAlgo.Cluster)(clusterList.elementAt(mClusterInView));
			mViewdCluster = auxCluster;
			
			int x = 0;
			double maxY = 0;
			double minY = 0;
			double y;
			
			boolean computeMinMax = false;
			
			if (computeMinMax){
				for (int i=0;i<auxCluster.mValues.length;i++){
					y = auxCluster.mValues[i];
		
					if (y < minY || i == 0)
						minY = y;
					
					if (y > maxY || i == 0)
						maxY = y;
				}
			}
			else{
				maxY = mClusterAlgo.GetSeriesMaxValue();
				minY = mClusterAlgo.GetSeriesMinValue();
			}

			int oldX = -1;
			int oldY = -1;
			
			int xLoc = 0;
			int yLoc = 0;
			xLoc = mDisplayRegion.x + mIntervalShift * dotSize;
			
			int numIntervals = auxCluster.mValues.length;
			mLeapX = mDisplayRegion.width / numIntervals;
			if (mLeapX < dotSize)
				mLeapX = dotSize;	
			
			for (int i=0;i<auxCluster.mValues.length;i++){
				
				if (xLoc >= mDisplayRegion.getMaxX())
					break;

				if (xLoc >=  mDisplayRegion.x){				
					y = auxCluster.mValues[i];
				
					// normalize Y
					y = (y-minY)/(maxY-minY);
				
					// draw something
					yLoc = (int)(mDisplayRegion.getMaxY() - mDisplayRegion.height * y);
					g2.fillRect(xLoc-halfDotSize,yLoc-halfDotSize,dotSize,dotSize);
					
					if (oldX >= 0 && oldY >= 0)
					{
						g2.drawLine(xLoc,yLoc,oldX,oldY);
					}

					oldX = xLoc;
					oldY = yLoc;

				}

				xLoc += mLeapX;
			}
			
			DrawAxes(g2,auxCluster.mValues.length);

			g2.dispose();
		}
		
		g2 = finalImg.createGraphics();
		g2.setBackground(Color.white);
		g2.drawImage(bimg,0,0,this);

	}
	
	private void DrawAxes(Graphics2D g2,int numIntervals)
	{
		int numTicks = 10;

		g2.setColor(Color.gray);
		g2.drawRect((int)mDisplayRegion.getMinX(),
					(int)mDisplayRegion.getMinY(),
					(int)mDisplayRegion.width,
					(int)mDisplayRegion.height);
	
		double yRatio = 0.0;
		
		g2.setColor(Color.black);

		double maxY = mClusterAlgo.GetSeriesMaxValue();
		double minY = mClusterAlgo.GetSeriesMinValue();
		int y = 0;
		
		NumberFormat numFormat = NumberFormat.getInstance();
		numFormat.setMinimumFractionDigits( 3 );
		numFormat.setMaximumFractionDigits( 3 );

		// what font size to use	

		if (mDrawTickLabels){ 
			Font smallFont = new Font("Serif",Font.PLAIN,mTickLabelFontSize);
			Font saveFont = g2.getFont();
			g2.setFont(smallFont);
	
			for (int i=0;i<numTicks;i++){
	
				y = (int)(mDisplayRegion.getMaxY()-(yRatio * mDisplayRegion.height));
				g2.drawLine(mDisplayRegion.x-3,y,mDisplayRegion.x,y);
	
				double valueInTick = yRatio * (maxY-minY)+minY;
				
				String valStr = numFormat.format( valueInTick );
				g2.drawString(valStr,1,y);
				
				yRatio += 1.0/(double)numTicks;
			}
	
			g2.setFont(saveFont);
		}

		g2.setColor(Color.black);
		int x = mDisplayRegion.x;
		
		Dimension d = getSize();


		numTicks = numIntervals;		

		for (int i=0;i<numTicks;i++){
			g2.drawLine(x,(int)mDisplayRegion.getMaxY()+3,x,(int)mDisplayRegion.getMaxY());
			x += mLeapX;
		}
	}
	
	protected void DrawPointInfo(Point p)
	{
		if (mLeapX < 0 || mViewdCluster == null)
			return;
			
		
		Graphics2D g2 = (Graphics2D)finalImg.createGraphics();
		g2.drawImage(bimg,0,0,this);
		
		if (mDisplayRegion.contains(p) && mViewdCluster != null){
			
			System.out.println("DrawPointInfo");
			
			int intervalIndex = (int)((double)(p.x - mDisplayRegion.x) / mLeapX) + 1;

			System.out.println(intervalIndex);

	//		double xRatio = (double)(p.x - mDisplayRegion.x)/(double)mDisplayRegion.width;
			double y = 0.0;
			
			double maxY = mClusterAlgo.GetSeriesMaxValue();
			double minY = mClusterAlgo.GetSeriesMinValue();
						
			if (intervalIndex >= 0 && intervalIndex < mViewdCluster.mValues.length){
				y = mViewdCluster.mValues[intervalIndex];
			}

			double yRatio = (y-minY)/(maxY-minY);
		
			//int locX = (int)(mDisplayRegion.width * xRatio + mDisplayRegion.x);
			
			int locX = (intervalIndex-1)*mLeapX + mDisplayRegion.x;
			int locY = (int)(mDisplayRegion.getMaxY()-mDisplayRegion.height * yRatio);
			
	//		g2.drawLine(locX,(int)mDisplayRegion.getMaxY()+5,locX,(int)mDisplayRegion.getMaxY());
	//		g2.drawLine(mDisplayRegion.x-5,locY,mDisplayRegion.x,locY);

			g2.setColor(Color.lightGray);
			g2.drawLine(locX,(int)mDisplayRegion.getMaxY(),locX,locY);
			g2.drawLine((int)mDisplayRegion.x,locY,(int)mDisplayRegion.x-5,locY);

			g2.setColor(Color.black);
	
			Dimension d = getSize();
			NumberFormat numFormat = NumberFormat.getInstance();
			numFormat.setMinimumFractionDigits( 3 );
			numFormat.setMaximumFractionDigits( 3 );
			
			FontMetrics fm   = g2.getFontMetrics();  // metrics for this object

			String valStr = "";
			valStr  += intervalIndex;
			Rectangle2D rect = fm.getStringBounds(valStr, g2); // size of string
			int strWidth = (int)rect.getWidth();
			if (locX + strWidth > d.width)
				locX -= (locX + strWidth - d.width);
			g2.drawString(valStr,locX,(int)mDisplayRegion.getMaxY()+15);
			
			valStr = numFormat.format( y );

			rect = fm.getStringBounds(valStr, g2); // size of string
			strWidth = (int)rect.getWidth();
			if (locY < strWidth)
				locY = strWidth;

			g2.translate(mDisplayRegion.x-8,locY);
			g2.rotate(-Math.PI/2);
			g2.drawString(valStr,0,0);
		}			
		g2.dispose();
		Graphics2D g = (Graphics2D) getGraphics();
		g.drawImage(finalImg, 0, 0, this);
	}
	
	public void DoRepaint()
	{
		Graphics2D g = (Graphics2D) getGraphics();
		PaintBufferedImg();
		g.drawImage(finalImg, 0, 0, this);
	}
	
	public void actionPerformed( ActionEvent event )
	{
		String cmdString = event.getActionCommand();
		
		if (mClusterAlgo == null){
			mClusterInView = 0;
			return;
		}

		boolean needRepaint = false;
		
		Vector clusterList = mClusterAlgo.ExtractClusters();
		if (clusterList == null){
			System.out.println("No cluster list");
			return;
		}
		
		if (cmdString == BTN_LEFT){
			mClusterInView -= 1;
			if (mClusterInView < 0){
				mClusterInView = clusterList.size()-1;	
			}
			needRepaint = true;
		}
		else if (cmdString == BTN_RIGHT){
			mClusterInView += 1;
			if (mClusterInView >= clusterList.size()){
				mClusterInView = 0;	
			}
			needRepaint = true;
		}
	
	if (needRepaint)
		DoRepaint();
	}
	
	
	public void keyPressed( KeyEvent e ) 
	{
		int key = e.getKeyCode();
		boolean consumed = false;
		boolean isCtrlClicked = ((e.getModifiers() & InputEvent.CTRL_MASK) != 0);


		if (key == KeyEvent.VK_LEFT){

			mIntervalShift += 1;
			repaint();

			consumed = true;

		}
		else if (key == KeyEvent.VK_RIGHT){


			mIntervalShift -= 1;
			repaint();
			consumed = true;
		}
				
		if (consumed)
			e.consume();
	}
	
	public void keyReleased( KeyEvent e ) {}
	public void keyTyped( KeyEvent e ) {}
	
	public void mouseMoved(MouseEvent e) {
		DrawPointInfo(e.getPoint());
	}	
	
	public void mousePressed(MouseEvent e) {
		requestFocus();
	 }

	 public void mouseReleased(MouseEvent e) {
	 }

	 public void mouseEntered(MouseEvent e) {
	 }

	 public void mouseExited(MouseEvent e) {
		DrawPointInfo(new Point(-1,-1));
	 }
	 
	public void mouseDragged(MouseEvent e) {

	}	
	 

	 public void mouseClicked(MouseEvent e) {
	
		Dimension d = getSize();
		Point p = e.getPoint();
		Rectangle leftButtonBounds = leftButton.getBounds();
		if (e.getClickCount() == 2){
			if (mSavedHeight < 0){
				mSavedHeight = d.height;
				mDlg.setSize(new Dimension(d.width,50));
				mDlg.repaint();
			}
			else{
				mDlg.setSize(new Dimension(d.width,mSavedHeight));
				mDlg.repaint();
				mSavedHeight = -1;
			}
		}
	 }
	 
	public void componentHidden(ComponentEvent e) {
		if (mVizMgr != null)
			mVizMgr.SetShowClusterViewer(false);
	}

	public void componentMoved(ComponentEvent e) {
		Component c = e.getComponent();
	}

	public void componentResized(ComponentEvent e) {
		Component c = e.getComponent();
		HandlePanelResize();					   
	}

	public void componentShown(ComponentEvent e) {
	}
	
	protected void HandlePanelResize()
	{
		Dimension d = getSize();
		mDisplayRegion = new Rectangle(40,5,d.width-45,d.height-35);


		// will labels be displayed ?
		
		int numTicks = 10;
		mDrawTickLabels = true;
		int fontSize = (int)(mDisplayRegion.height * 0.8 / numTicks);
		if (fontSize > 10){
			fontSize = 10;
		}
		else if (fontSize <= 4){
			mDrawTickLabels = false;
		}
		
		mTickLabelFontSize = fontSize;

		if (!mDrawTickLabels)
		{
			mDisplayRegion = new Rectangle(10,5,d.width-7,d.height-10);
		}
		rightButton.setLocation(10,d.height-10);
		leftButton.setLocation(0,d.height-10);
	}
	 

	JDialog CreateContainerDialog(JFrame ownerFrame)
	{
		if (mDlg != null)
			return mDlg;
			
		mDlg = new JDialog(ownerFrame,false);
		mDlg.setLocationRelativeTo(ownerFrame);
		mDlg.setTitle("Cluster Viewer");
		mDlg.setSize(new Dimension(300,250));
		//mDlg.setDefaultCloseOperatio(HIDE_ON_CLOSE );
		mDlg.addComponentListener(this);
		
		this.setPreferredSize(new Dimension(300,250));
//		dlg.getContentPane().setLayout(new OverlayLayout());
		mDlg.getContentPane().add(this);
		return mDlg;
	}
	
}
