/*
 * Created on Apr 16, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package Viz;

import java.awt.*;
import javax.swing.*;
import java.awt.image.BufferedImage;
import java.util.*;
import java.awt.event.*;
import java.text.NumberFormat;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;

import DataProcess.*;

/**
 * @author Lior Berry
 * UBC
*/
public class ScatterPlotPanel extends JPanel implements MouseListener,MouseMotionListener,ComponentListener{
	
	protected ArrayList mXValues = null;
	protected ArrayList mYValues = null;
	
	protected double maxX,minX;
	protected double maxY,minY;
	
	
	private BufferedImage bimg = null;
	protected Rectangle mDisplayRegion;
	
	private int dotSize = 3;
	private int halfDotSize = 1;
	
	private DataSet mDataSet;
	
	protected JDialog mDlg = null;
	protected VizPanelMgr		mVizMgr = null;
	protected int mSavedHeight = -1;
	
	
	
	Color mCurrencyColor[] = {Color.blue,Color.red};
	
	double mCorrIntercept 	= 0.0;
	double mCorrSlope		= 0.0;
	
	public void RegisterWithVizPanelMgr(VizPanelMgr inMgr)
	{
		mVizMgr = inMgr;
	}
	
	public void SetValues(ArrayList xValues,
						  int currencyIndex_X,
						  ArrayList yValues,
						  int currencyIndex_Y
						  )
	{
		mXValues = xValues;
		mYValues = yValues;
		
		TimeSeries ts;
		ts = mDataSet.GetSeriesByIndex(currencyIndex_X);
		maxX = ts.GetMax();
		minX = ts.GetMin();

		ts = mDataSet.GetSeriesByIndex(currencyIndex_Y);
		maxY = ts.GetMax();
		minY = ts.GetMin();
	}

	private Graphics2D createGraphics2D(int w, int h) {
		
		Graphics2D g2 = null;
		if (bimg == null || bimg.getWidth() != w || bimg.getHeight() != h) {
			bimg = (BufferedImage) createImage(w, h);
		} 
		g2 = bimg.createGraphics();
		g2.setBackground(Color.white);
		g2.clearRect(0, 0, w, h);
		return g2;
	}	
	
	public void paintComponent(Graphics gfx) {
		
		super.paintComponent(gfx);
		Graphics2D g = (Graphics2D) gfx;
		PaintBufferedImg();
		g.drawImage(bimg, 0, 0, this);
	}
	
	public void Init()
	{
		Dimension d = getSize();
		
		// set display region
		HandlePanelResize();

		// register for tooltips		
		ToolTipManager.sharedInstance().registerComponent(this);

		// add listerner
		addMouseMotionListener(this);
		addMouseListener(this);
	}
	
	public void AttachDataSet(DataSet inDataSet,boolean doRepaint)
	{
		mDataSet = inDataSet;
		if (bimg != null && doRepaint){
			DoRepaint();
		}
	}
	
	private void PaintBufferedImg()
	{
		Dimension d = getSize();
		
		Graphics2D g2 = createGraphics2D(d.width, d.height);
		
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				   RenderingHints.VALUE_ANTIALIAS_ON);
	
		if (mXValues == null || mYValues == null)
			return;


		boolean drawBorder = true;
		if (drawBorder){
			g2.setColor(Color.black);
			g2.drawRect(0,0,d.width-1,d.height-1);
		}
		BinData binVals = null;
	
		if (mXValues != null && mYValues != null){
			g2.setColor(Color.black);

			for (int i=0;i<mXValues.size();i++){

				binVals = (BinData)(mXValues.get(i));
				double x = binVals.mNormalizedValue;
				binVals = (BinData)(mYValues.get(i));
				double y = binVals.mNormalizedValue;

				// Adapt to panel size				
				int xLoc = (int)(mDisplayRegion.x + mDisplayRegion.width * x);
				int yLoc = (int)(mDisplayRegion.getMaxY() - mDisplayRegion.height * y);
				
				g2.drawLine(xLoc,yLoc-halfDotSize,xLoc,yLoc+halfDotSize);
				g2.drawLine(xLoc-halfDotSize,yLoc,xLoc+halfDotSize,yLoc);
		
				//g2.fillRect((int)x-1,(int)y-1,dotSize,dotSize);
			}
		}

		ComputeRegressionLine();
		double x1 = 0.0;
		double y1 = (int)(mDisplayRegion.getMaxY()-mCorrIntercept*mDisplayRegion.height);
		double x2 = 1.0;
		double y2 = (int)(mDisplayRegion.getMaxY()-mDisplayRegion.height*(mCorrIntercept+mCorrSlope*x2));


		x1 *= mDisplayRegion.width;
		x2 *= mDisplayRegion.width;

		g2.setColor(Color.red);
		g2.drawLine((int)(x1+mDisplayRegion.x),(int)y1,(int)(x2+mDisplayRegion.x),(int)y2);
		
		DrawAxes(g2);
		g2.dispose();
	}
	
	private void DrawAxes(Graphics2D g2)
	{
		int numTicks = 10;

		g2.setColor(Color.gray);
		g2.drawRect((int)mDisplayRegion.getMinX(),
					(int)mDisplayRegion.getMinY(),
					(int)mDisplayRegion.width,
					(int)mDisplayRegion.height);
	
		int y = (int)mDisplayRegion.getMaxY();

		g2.setColor(mCurrencyColor[1]);
		for (int i=0;i<numTicks;i++){
			g2.drawLine(mDisplayRegion.x-3,y,mDisplayRegion.x,y);
			y -= mDisplayRegion.height/numTicks;
		}

		g2.setColor(mCurrencyColor[0]);
		int x = mDisplayRegion.x;
		
		Dimension d = getSize();

		for (int i=0;i<numTicks;i++){
			g2.drawLine(x,(int)mDisplayRegion.getMaxY()+3,x,(int)mDisplayRegion.getMaxY());
			x += mDisplayRegion.width/numTicks;
		}
	}
	
	protected void DrawPointInfo(Point p)
	{
		DoRepaint();
		Graphics2D g2 = (Graphics2D)(getGraphics());
		if (mDisplayRegion.contains(p)){
			
			double xRatio = (double)(p.x - mDisplayRegion.x)/(double)mDisplayRegion.width;
			double yRatio = (double)(p.y - mDisplayRegion.y)/(double)mDisplayRegion.height;
			
			double xVal = (maxX - minX)*xRatio + minX;
			double yVal = (maxY - minY)*yRatio + minY;

			int locX = (int)(mDisplayRegion.width * xRatio + mDisplayRegion.x);
			int locY = (int)(mDisplayRegion.height * yRatio + mDisplayRegion.y);
			
			g2.setColor(Color.black);
			g2.drawLine(locX,(int)mDisplayRegion.getMaxY()+5,locX,(int)mDisplayRegion.getMaxY());
			g2.drawLine(mDisplayRegion.x-5,locY,mDisplayRegion.x,locY);

			Dimension d = getSize();
			NumberFormat numFormat = NumberFormat.getInstance();
			numFormat.setMinimumFractionDigits( 3 );
			numFormat.setMaximumFractionDigits( 3 );

			
			FontMetrics fm   = g2.getFontMetrics();  // metrics for this object

			String valStr = numFormat.format( xVal );
			Rectangle2D rect = fm.getStringBounds(valStr, g2); // size of string
			int strWidth = (int)rect.getWidth();
			if (locX + strWidth > d.width)
				locX -= (locX + strWidth - d.width);
				
			g2.setColor(mCurrencyColor[0]);
			g2.drawString(valStr,locX,d.height-2);
			
			g2.setColor(mCurrencyColor[1]);
			valStr = numFormat.format( yVal );

			rect = fm.getStringBounds(valStr, g2); // size of string
			strWidth = (int)rect.getWidth();
			if (locY < strWidth)
				locY = strWidth;

			g2.translate(10,locY);
			g2.rotate(-Math.PI/2);
			g2.drawString(valStr,0,0);
		}			
		g2.dispose();
	}
	
	protected void ComputeRegressionLine()
	{
		double sumX = 0.0;
		double sumY = 0.0;
		double sumXSquare = 0.0;
		double sumYSquare = 0.0;
		double sumXY = 0.0;
		
		BinData binVals;
		
		for (int i=0;i<mXValues.size();i++){

			binVals = (BinData)(mXValues.get(i));
			double x = binVals.mNormalizedValue;
			sumX += x;
			sumXSquare += x*x;

			binVals = (BinData)(mYValues.get(i));
			double y = binVals.mNormalizedValue;
			sumY += y;
			sumYSquare += y*y;
			
			sumXY += x*y;
		}
	
		int n = mXValues.size();

		mCorrSlope = (n*sumXY-sumX*sumY) / (n*sumXSquare - sumX*sumX);
		mCorrIntercept = (sumY - mCorrSlope*sumX)/n;
/*
		mCorrSlope = (n*sumXY - (sumX * sumY)) / (n*sumXSquare - sumX*sumX);
		mCorrIntercept = (sumY * sumXSquare - sumX * sumXY) / (n*sumXY-sumX*sumX);
*/	
		System.out.println("slope="+mCorrSlope);
		System.out.println("intercept="+mCorrIntercept);
	}
	
	public String getToolTipText(java.awt.event.MouseEvent event) 	
	{
		String ret = null;
		/*
		Point p = event.getPoint();
		
		if (mDisplayRegion.contains(p)){
			double xRatio = (double)(p.x - mDisplayRegion.x)/(double)mDisplayRegion.width;
			double yRatio = (double)(p.y - mDisplayRegion.y)/(double)mDisplayRegion.height;
			
			double xVal = (maxX - minX)*xRatio + minX;
			double yVal = (maxY - minY)*yRatio + minY;
			
			ret = new String(""+xVal+","+yVal);
							
		}	
		*/
		return ret;
	}
	
	public void DoRepaint()
	{
		Graphics2D g = (Graphics2D) getGraphics();
		PaintBufferedImg();
		g.drawImage(bimg, 0, 0, this);
	}
	
	public void SetColorForSeries(int seriesIndex,Color inColor)
	{
		mCurrencyColor[seriesIndex] = inColor;
	}
	
	public void mouseMoved(MouseEvent e) {
		DrawPointInfo(e.getPoint());
	}	
	
	public void mouseDragged(MouseEvent e) {

	}	
	
	public void mousePressed(MouseEvent e) {
	 }

	 public void mouseReleased(MouseEvent e) {
	 }

	 public void mouseEntered(MouseEvent e) {
	 }

	 public void mouseExited(MouseEvent e) {
	 	DrawPointInfo(new Point(-1,-1));
	 }

	 public void mouseClicked(MouseEvent e) {
		Dimension d = getSize();
		Point p = e.getPoint();
		if (e.getClickCount() == 2){
			if (mSavedHeight < 0){
				mSavedHeight = d.height;
				mDlg.setSize(new Dimension(d.width,50));
				mDlg.repaint();
			}
			else{
				mDlg.setSize(new Dimension(d.width,mSavedHeight));
				mDlg.repaint();
				mSavedHeight = -1;
			}
		}
	 }


	public void componentHidden(ComponentEvent e) {
		if (mVizMgr != null)
			mVizMgr.SetShowScatterPlot(false);
	}

	public void componentMoved(ComponentEvent e) {
		Component c = e.getComponent();
	}

	public void componentResized(ComponentEvent e) {
		Component c = e.getComponent();
		HandlePanelResize();					   
	}

	public void componentShown(ComponentEvent e) {
	}
	
	protected void HandlePanelResize()
	{
		Dimension d = getSize();
		mDisplayRegion = new Rectangle(20,0,d.width-10,d.height-20);
	}

	
	JDialog CreateContainerDialog(JFrame ownerFrame)
	{
		if (mDlg != null)
			return mDlg;
			
		mDlg = new JDialog(ownerFrame,false);
		mDlg.setLocationRelativeTo(ownerFrame);
		mDlg.setTitle("Scatter Plot Viewer");
		mDlg.setSize(new Dimension(300,250));
		//mDlg.setDefaultCloseOperatio(HIDE_ON_CLOSE );
		mDlg.addComponentListener(this);
		
		this.setPreferredSize(new Dimension(300,250));
//		dlg.getContentPane().setLayout(new OverlayLayout());
		mDlg.getContentPane().add(this);
		return mDlg;
	}	
}
